# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_output import WlOutput
from .wl_seat import WlSeat
from .wl_surface import WlSurface


class WlShellSurface(Interface):
    """Desktop-style metadata interface

    An interface that may be implemented by a
    :class:`~pywayland.protocol.wayland.WlSurface`, for implementations that
    provide a desktop-style user interface.

    It provides requests to treat surfaces like toplevel, fullscreen or popup
    windows, move, resize or maximize them, associate metadata like title and
    class, etc.

    On the server side the object is automatically destroyed when the related
    :class:`~pywayland.protocol.wayland.WlSurface` is destroyed. On the client
    side, wl_shell_surface_destroy() must be called before destroying the
    :class:`~pywayland.protocol.wayland.WlSurface` object.
    """

    name = "wl_shell_surface"
    version = 1

    class resize(enum.IntFlag):
        none = 0
        top = 1
        bottom = 2
        left = 4
        top_left = 5
        bottom_left = 6
        right = 8
        top_right = 9
        bottom_right = 10

    class transient(enum.IntFlag):
        inactive = 0x1

    class fullscreen_method(enum.IntEnum):
        default = 0
        scale = 1
        driver = 2
        fill = 3


class WlShellSurfaceProxy(Proxy):
    interface = WlShellSurface

    @WlShellSurface.request(
        Argument(ArgumentType.Uint),
    )
    def pong(self, serial):
        """Respond to a ping event

        A client must respond to a ping event with a pong request or the client
        may be deemed unresponsive.

        :param serial:
            serial number of the ping event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(0, serial)

    @WlShellSurface.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
    )
    def move(self, seat, serial):
        """Start an interactive move

        Start a pointer-driven move of the surface.

        This request must be used in response to a button press event. The
        server may ignore move requests depending on the state of the surface
        (e.g. fullscreen or maximized).

        :param seat:
            seat whose pointer is used
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            serial number of the implicit grab on the pointer
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(1, seat, serial)

    @WlShellSurface.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def resize(self, seat, serial, edges):
        """Start an interactive resize

        Start a pointer-driven resizing of the surface.

        This request must be used in response to a button press event. The
        server may ignore resize requests depending on the state of the surface
        (e.g. fullscreen or maximized).

        :param seat:
            seat whose pointer is used
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            serial number of the implicit grab on the pointer
        :type serial:
            `ArgumentType.Uint`
        :param edges:
            which edge or corner is being dragged
        :type edges:
            `ArgumentType.Uint`
        """
        self._marshal(2, seat, serial, edges)

    @WlShellSurface.request()
    def set_toplevel(self):
        """Make the surface a toplevel surface

        Map the surface as a toplevel surface.

        A toplevel surface is not fullscreen, maximized or transient.
        """
        self._marshal(3)

    @WlShellSurface.request(
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Uint),
    )
    def set_transient(self, parent, x, y, flags):
        """Make the surface a transient surface

        Map the surface relative to an existing surface.

        The x and y arguments specify the location of the upper left corner of
        the surface relative to the upper left corner of the parent surface, in
        surface-local coordinates.

        The flags argument controls details of the transient behaviour.

        :param parent:
            parent surface
        :type parent:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param x:
            surface-local x coordinate
        :type x:
            `ArgumentType.Int`
        :param y:
            surface-local y coordinate
        :type y:
            `ArgumentType.Int`
        :param flags:
            transient surface behavior
        :type flags:
            `ArgumentType.Uint`
        """
        self._marshal(4, parent, x, y, flags)

    @WlShellSurface.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlOutput, nullable=True),
    )
    def set_fullscreen(self, method, framerate, output):
        """Make the surface a fullscreen surface

        Map the surface as a fullscreen surface.

        If an output parameter is given then the surface will be made
        fullscreen on that output. If the client does not specify the output
        then the compositor will apply its policy - usually choosing the output
        on which the surface has the biggest surface area.

        The client may specify a method to resolve a size conflict between the
        output size and the surface size - this is provided through the method
        parameter.

        The framerate parameter is used only when the method is set to
        "driver", to indicate the preferred framerate. A value of 0 indicates
        that the client does not care about framerate.  The framerate is
        specified in mHz, that is framerate of 60000 is 60Hz.

        A method of "scale" or "driver" implies a scaling operation of the
        surface, either via a direct scaling operation or a change of the
        output mode. This will override any kind of output scaling, so that
        mapping a surface with a buffer size equal to the mode can fill the
        screen independent of buffer_scale.

        A method of "fill" means we don't scale up the buffer, however any
        output scale is applied. This means that you may run into an edge case
        where the application maps a buffer with the same size of the output
        mode but buffer_scale 1 (thus making a surface larger than the output).
        In this case it is allowed to downscale the results to fit the screen.

        The compositor must reply to this request with a configure event with
        the dimensions for the output on which the surface will be made
        fullscreen.

        :param method:
            method for resolving size conflict
        :type method:
            `ArgumentType.Uint`
        :param framerate:
            framerate in mHz
        :type framerate:
            `ArgumentType.Uint`
        :param output:
            output on which the surface is to be fullscreen
        :type output:
            :class:`~pywayland.protocol.wayland.WlOutput` or `None`
        """
        self._marshal(5, method, framerate, output)

    @WlShellSurface.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Uint),
    )
    def set_popup(self, seat, serial, parent, x, y, flags):
        """Make the surface a popup surface

        Map the surface as a popup.

        A popup surface is a transient surface with an added pointer grab.

        An existing implicit grab will be changed to owner-events mode, and the
        popup grab will continue after the implicit grab ends (i.e. releasing
        the mouse button does not cause the popup to be unmapped).

        The popup grab continues until the window is destroyed or a mouse
        button is pressed in any other client's window. A click in any of the
        client's surfaces is reported as normal, however, clicks in other
        clients' surfaces will be discarded and trigger the callback.

        The x and y arguments specify the location of the upper left corner of
        the surface relative to the upper left corner of the parent surface, in
        surface-local coordinates.

        :param seat:
            seat whose pointer is used
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            serial number of the implicit grab on the pointer
        :type serial:
            `ArgumentType.Uint`
        :param parent:
            parent surface
        :type parent:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param x:
            surface-local x coordinate
        :type x:
            `ArgumentType.Int`
        :param y:
            surface-local y coordinate
        :type y:
            `ArgumentType.Int`
        :param flags:
            transient surface behavior
        :type flags:
            `ArgumentType.Uint`
        """
        self._marshal(6, seat, serial, parent, x, y, flags)

    @WlShellSurface.request(
        Argument(ArgumentType.Object, interface=WlOutput, nullable=True),
    )
    def set_maximized(self, output):
        """Make the surface a maximized surface

        Map the surface as a maximized surface.

        If an output parameter is given then the surface will be maximized on
        that output. If the client does not specify the output then the
        compositor will apply its policy - usually choosing the output on which
        the surface has the biggest surface area.

        The compositor will reply with a configure event telling the expected
        new surface size. The operation is completed on the next buffer attach
        to this surface.

        A maximized surface typically fills the entire output it is bound to,
        except for desktop elements such as panels. This is the main difference
        between a maximized shell surface and a fullscreen shell surface.

        The details depend on the compositor implementation.

        :param output:
            output on which the surface is to be maximized
        :type output:
            :class:`~pywayland.protocol.wayland.WlOutput` or `None`
        """
        self._marshal(7, output)

    @WlShellSurface.request(
        Argument(ArgumentType.String),
    )
    def set_title(self, title):
        """Set surface title

        Set a short title for the surface.

        This string may be used to identify the surface in a task bar, window
        list, or other user interface elements provided by the compositor.

        The string must be encoded in UTF-8.

        :param title:
            surface title
        :type title:
            `ArgumentType.String`
        """
        self._marshal(8, title)

    @WlShellSurface.request(
        Argument(ArgumentType.String),
    )
    def set_class(self, class_):
        """Set surface class

        Set a class for the surface.

        The surface class identifies the general class of applications to which
        the surface belongs. A common convention is to use the file name (or
        the full path if it is a non-standard location) of the application's
        .desktop file as the class.

        :param class_:
            surface class
        :type class_:
            `ArgumentType.String`
        """
        self._marshal(9, class_)


class WlShellSurfaceResource(Resource):
    interface = WlShellSurface

    @WlShellSurface.event(
        Argument(ArgumentType.Uint),
    )
    def ping(self, serial):
        """Ping client

        Ping a client to check if it is receiving events and sending requests.
        A client is expected to reply with a pong request.

        :param serial:
            serial number of the ping
        :type serial:
            `ArgumentType.Uint`
        """
        self._post_event(0, serial)

    @WlShellSurface.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def configure(self, edges, width, height):
        """Suggest resize

        The configure event asks the client to resize its surface.

        The size is a hint, in the sense that the client is free to ignore it
        if it doesn't resize, pick a smaller size (to satisfy aspect ratio or
        resize in steps of NxM pixels).

        The edges parameter provides a hint about how the surface was resized.
        The client may use this information to decide how to adjust its content
        to the new size (e.g. a scrolling area might adjust its content
        position to leave the viewable content unmoved).

        The client is free to dismiss all but the last configure event it
        received.

        The width and height arguments specify the size of the window in
        surface-local coordinates.

        :param edges:
            how the surface was resized
        :type edges:
            `ArgumentType.Uint`
        :param width:
            new width of the surface
        :type width:
            `ArgumentType.Int`
        :param height:
            new height of the surface
        :type height:
            `ArgumentType.Int`
        """
        self._post_event(1, edges, width, height)

    @WlShellSurface.event()
    def popup_done(self):
        """Popup interaction is done

        The popup_done event is sent out when a popup grab is broken, that is,
        when the user clicks a surface that doesn't belong to the client owning
        the popup surface.
        """
        self._post_event(2)


class WlShellSurfaceGlobal(Global):
    interface = WlShellSurface


WlShellSurface._gen_c()
WlShellSurface.proxy_class = WlShellSurfaceProxy
WlShellSurface.resource_class = WlShellSurfaceResource
WlShellSurface.global_class = WlShellSurfaceGlobal



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
from typing import Optional
import enum
import xml.etree.ElementTree as ET

from .description import Description
from .element import Element
from .printer import Printer

NO_IFACE_NAME = "interface"


@enum.unique
class ArgumentType(enum.Enum):
    Int = enum.auto()
    Uint = enum.auto()
    Fixed = enum.auto()
    String = enum.auto()
    Object = enum.auto()
    NewId = enum.auto()
    Array = enum.auto()
    FileDescriptor = enum.auto()


@dataclass(frozen=True)
class Argument(Element):
    """Argument to a request or event method

    Required attributes: `name` and `type`

    Optional attributes: `summary`, `interface`, and `allow-null`

    Child elements: `description`
    """

    name: str
    type: ArgumentType
    summary: Optional[str]
    interface: Optional[str]
    allow_null: bool
    enum: Optional[str]
    description: Optional[Description]

    @classmethod
    def parse(cls, element: ET.Element) -> "Argument":
        arg_type_str = cls.parse_attribute(element, "type")
        if arg_type_str == "int":
            argument_type = ArgumentType.Int
        elif arg_type_str == "uint":
            argument_type = ArgumentType.Uint
        elif arg_type_str == "fixed":
            argument_type = ArgumentType.Fixed
        elif arg_type_str == "string":
            argument_type = ArgumentType.String
        elif arg_type_str == "object":
            argument_type = ArgumentType.Object
        elif arg_type_str == "new_id":
            argument_type = ArgumentType.NewId
        elif arg_type_str == "array":
            argument_type = ArgumentType.Array
        elif arg_type_str == "fd":
            argument_type = ArgumentType.FileDescriptor
        else:
            raise ValueError(f"Invalid argument type: {argument_type}")

        allow_null = cls.parse_optional_attribute(element, "allow-null") == "true"
        return cls(
            name=cls.parse_attribute(element, "name"),
            type=argument_type,
            summary=cls.parse_optional_attribute(element, "summary"),
            interface=cls.parse_optional_attribute(element, "interface"),
            allow_null=allow_null,
            enum=cls.parse_optional_attribute(element, "enum"),
            description=cls.parse_optional_child(element, Description, "description"),
        )

    @property
    def interface_class(self) -> str:
        """Returns the Interface class name

        Gives the class name for the Interface coresponding to the type of the
        argument.
        """
        assert self.interface is not None
        return "".join(x.capitalize() for x in self.interface.split("_"))

    def output(self, printer: Printer) -> None:
        args = [f"ArgumentType.{self.type.name}"]
        if self.interface is not None:
            args.append(f"interface={self.interface_class}")
        if self.allow_null:
            args.append("nullable=True")

        printer(f"Argument({', '.join(args)}),")

    def output_doc_param(self, printer: Printer) -> None:
        """Document the argument as a parameter"""
        # Output the parameter and summary
        printer.doc(":param {}:".format(self.name))
        if self.summary:
            with printer.indented():
                printer.docstring(self.summary)

        # Determine the type to be output
        if self.interface:
            arg_type = self.interface
        else:
            arg_type = "`{}`".format(self.type)

        # Output the parameter type
        printer.doc(f":type {self.name}:")
        with printer.indented():
            if self.allow_null:
                printer.doc(f"{arg_type} or `None`")
            else:
                printer.doc(f"{arg_type}")

    def output_doc_ret(self, printer: Printer) -> None:
        """Document the argument as a return"""
        # Determine the type to be output
        if self.interface:
            arg_type = self.interface
        else:
            # Only new_id's are returned, the only corner case here is for
            # wl_registry.bind, so no interface => Proxy
            arg_type = ":class:`pywayland.client.proxy.Proxy` of specified Interface"

        # Output the type and summary
        printer.doc(":returns:")
        with printer.indented():
            if self.summary:
                printer.docstring("{} -- {}".format(arg_type, self.summary))
            else:
                printer.docstring("{}".format(arg_type))



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_surface import WlSurface


class WlSubsurface(Interface):
    """Sub-surface interface to a :class:`~pywayland.protocol.wayland.WlSurface`

    An additional interface to a :class:`~pywayland.protocol.wayland.WlSurface`
    object, which has been made a sub-surface. A sub-surface has one parent
    surface. A sub-surface's size and position are not limited to that of the
    parent. Particularly, a sub-surface is not automatically clipped to its
    parent's area.

    A sub-surface becomes mapped, when a non-NULL
    :class:`~pywayland.protocol.wayland.WlBuffer` is applied and the parent
    surface is mapped. The order of which one happens first is irrelevant. A
    sub-surface is hidden if the parent becomes hidden, or if a NULL
    :class:`~pywayland.protocol.wayland.WlBuffer` is applied. These rules apply
    recursively through the tree of surfaces.

    The behaviour of a :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>` request on a sub-surface
    depends on the sub-surface's mode. The possible modes are synchronized and
    desynchronized, see methods :func:`WlSubsurface.set_sync()` and
    :func:`WlSubsurface.set_desync()`. Synchronized mode caches the
    :class:`~pywayland.protocol.wayland.WlSurface` state to be applied when the
    parent's state gets applied, and desynchronized mode applies the pending
    :class:`~pywayland.protocol.wayland.WlSurface` state directly. A sub-
    surface is initially in the synchronized mode.

    Sub-surfaces also have another kind of state, which is managed by
    :class:`WlSubsurface` requests, as opposed to
    :class:`~pywayland.protocol.wayland.WlSurface` requests. This state
    includes the sub-surface position relative to the parent surface
    (:func:`WlSubsurface.set_position()`), and the stacking order of the parent
    and its sub-surfaces (:func:`WlSubsurface.place_above()` and .place_below).
    This state is applied when the parent surface's
    :class:`~pywayland.protocol.wayland.WlSurface` state is applied, regardless
    of the sub-surface's mode. As the exception, set_sync and set_desync are
    effective immediately.

    The main surface can be thought to be always in desynchronized mode, since
    it does not have a parent in the sub-surfaces sense.

    Even if a sub-surface is in desynchronized mode, it will behave as in
    synchronized mode, if its parent surface behaves as in synchronized mode.
    This rule is applied recursively throughout the tree of surfaces. This
    means, that one can set a sub-surface into synchronized mode, and then
    assume that all its child and grand-child sub-surfaces are synchronized,
    too, without explicitly setting them.

    If the :class:`~pywayland.protocol.wayland.WlSurface` associated with the
    :class:`WlSubsurface` is destroyed, the :class:`WlSubsurface` object
    becomes inert. Note, that destroying either object takes effect
    immediately. If you need to synchronize the removal of a sub-surface to the
    parent surface update, unmap the sub-surface first by attaching a NULL
    :class:`~pywayland.protocol.wayland.WlBuffer`, update parent, and then
    destroy the sub-surface.

    If the parent :class:`~pywayland.protocol.wayland.WlSurface` object is
    destroyed, the sub-surface is unmapped.
    """

    name = "wl_subsurface"
    version = 1

    class error(enum.IntEnum):
        bad_surface = 0


class WlSubsurfaceProxy(Proxy):
    interface = WlSubsurface

    @WlSubsurface.request()
    def destroy(self):
        """Remove sub-surface interface

        The sub-surface interface is removed from the
        :class:`~pywayland.protocol.wayland.WlSurface` object that was turned
        into a sub-surface with a :func:`WlSubcompositor.get_subsurface()
        <pywayland.protocol.wayland.WlSubcompositor.get_subsurface>` request.
        The wl_surface's association to the parent is deleted, and the
        :class:`~pywayland.protocol.wayland.WlSurface` loses its role as a sub-
        surface. The :class:`~pywayland.protocol.wayland.WlSurface` is unmapped
        immediately.
        """
        self._marshal(0)
        self._destroy()

    @WlSubsurface.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_position(self, x, y):
        """Reposition the sub-surface

        This schedules a sub-surface position change. The sub-surface will be
        moved so that its origin (top left corner pixel) will be at the
        location x, y of the parent surface coordinate system. The coordinates
        are not restricted to the parent surface area. Negative values are
        allowed.

        The scheduled coordinates will take effect whenever the state of the
        parent surface is applied. When this happens depends on whether the
        parent surface is in synchronized mode or not. See
        :func:`WlSubsurface.set_sync()` and :func:`WlSubsurface.set_desync()`
        for details.

        If more than one set_position request is invoked by the client before
        the commit of the parent surface, the position of a new request always
        replaces the scheduled position from any previous request.

        The initial position is 0, 0.

        :param x:
            x coordinate in the parent surface
        :type x:
            `ArgumentType.Int`
        :param y:
            y coordinate in the parent surface
        :type y:
            `ArgumentType.Int`
        """
        self._marshal(1, x, y)

    @WlSubsurface.request(
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def place_above(self, sibling):
        """Restack the sub-surface

        This sub-surface is taken from the stack, and put back just above the
        reference surface, changing the z-order of the sub-surfaces. The
        reference surface must be one of the sibling surfaces, or the parent
        surface. Using any other surface, including this sub-surface, will
        cause a protocol error.

        The z-order is double-buffered. Requests are handled in order and
        applied immediately to a pending state. The final pending state is
        copied to the active state the next time the state of the parent
        surface is applied. When this happens depends on whether the parent
        surface is in synchronized mode or not. See
        :func:`WlSubsurface.set_sync()` and :func:`WlSubsurface.set_desync()`
        for details.

        A new sub-surface is initially added as the top-most in the stack of
        its siblings and parent.

        :param sibling:
            the reference surface
        :type sibling:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._marshal(2, sibling)

    @WlSubsurface.request(
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def place_below(self, sibling):
        """Restack the sub-surface

        The sub-surface is placed just below the reference surface. See
        :func:`WlSubsurface.place_above()`.

        :param sibling:
            the reference surface
        :type sibling:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._marshal(3, sibling)

    @WlSubsurface.request()
    def set_sync(self):
        """Set sub-surface to synchronized mode

        Change the commit behaviour of the sub-surface to synchronized mode,
        also described as the parent dependent mode.

        In synchronized mode, :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` on a sub-surface will
        accumulate the committed state in a cache, but the state will not be
        applied and hence will not change the compositor output. The cached
        state is applied to the sub-surface immediately after the parent
        surface's state is applied. This ensures atomic updates of the parent
        and all its synchronized sub-surfaces. Applying the cached state will
        invalidate the cache, so further parent surface commits do not
        (re-)apply old state.

        See :class:`WlSubsurface` for the recursive effect of this mode.
        """
        self._marshal(4)

    @WlSubsurface.request()
    def set_desync(self):
        """Set sub-surface to desynchronized mode

        Change the commit behaviour of the sub-surface to desynchronized mode,
        also described as independent or freely running mode.

        In desynchronized mode, :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` on a sub-surface will
        apply the pending state directly, without caching, as happens normally
        with a :class:`~pywayland.protocol.wayland.WlSurface`. Calling
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` on the parent surface
        has no effect on the sub-surface's
        :class:`~pywayland.protocol.wayland.WlSurface` state. This mode allows
        a sub-surface to be updated on its own.

        If cached state exists when :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` is called in
        desynchronized mode, the pending state is added to the cached state,
        and applied as a whole. This invalidates the cache.

        Note: even if a sub-surface is set to desynchronized, a parent sub-
        surface may override it to behave as synchronized. For details, see
        :class:`WlSubsurface`.

        If a surface's parent surface behaves as desynchronized, then the
        cached state is applied on set_desync.
        """
        self._marshal(5)


class WlSubsurfaceResource(Resource):
    interface = WlSubsurface


class WlSubsurfaceGlobal(Global):
    interface = WlSubsurface


WlSubsurface._gen_c()
WlSubsurface.proxy_class = WlSubsurfaceProxy
WlSubsurface.resource_class = WlSubsurfaceResource
WlSubsurface.global_class = WlSubsurfaceGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright © 2017 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class ZxdgOutputV1(Interface):
    """Compositor logical output region

    An xdg_output describes part of the compositor geometry.

    This typically corresponds to a monitor that displays part of the
    compositor space.

    For objects version 3 onwards, after all xdg_output properties have been
    sent (when the object is created and when properties are updated), a
    :func:`WlOutput.done() <pywayland.protocol.wayland.WlOutput.done>` event is
    sent. This allows changes to the output properties to be seen as atomic,
    even if they happen via multiple events.
    """

    name = "zxdg_output_v1"
    version = 3


class ZxdgOutputV1Proxy(Proxy):
    interface = ZxdgOutputV1

    @ZxdgOutputV1.request()
    def destroy(self):
        """Destroy the xdg_output object

        Using this request a client can tell the server that it is not going to
        use the xdg_output object anymore.
        """
        self._marshal(0)
        self._destroy()


class ZxdgOutputV1Resource(Resource):
    interface = ZxdgOutputV1

    @ZxdgOutputV1.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def logical_position(self, x, y):
        """Position of the output within the global compositor space

        The position event describes the location of the
        :class:`~pywayland.protocol.wayland.WlOutput` within the global
        compositor space.

        The logical_position event is sent after creating an xdg_output (see
        xdg_output_manager.get_xdg_output) and whenever the location of the
        output changes within the global compositor space.

        :param x:
            x position within the global compositor space
        :type x:
            `ArgumentType.Int`
        :param y:
            y position within the global compositor space
        :type y:
            `ArgumentType.Int`
        """
        self._post_event(0, x, y)

    @ZxdgOutputV1.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def logical_size(self, width, height):
        """Size of the output in the global compositor space

        The logical_size event describes the size of the output in the global
        compositor space.

        For example, a surface without any buffer scale, transformation nor
        rotation set, with the size matching the logical_size will have the
        same size as the corresponding output when displayed.

        Most regular Wayland clients should not pay attention to the logical
        size and would rather rely on
        :class:`~pywayland.protocol.xdg_shell_unstable_v5.XdgShell` interfaces.

        Some clients such as Xwayland, however, need this to configure their
        surfaces in the global compositor space as the compositor may apply a
        different scale from what is advertised by the output scaling property
        (to achieve fractional scaling, for example).

        For example, for a :class:`~pywayland.protocol.wayland.WlOutput` mode
        3840×2160 and a scale factor 2:

        - A compositor not scaling the surface buffers will advertise a logical
          size of 3840×2160,

        - A compositor automatically scaling the surface buffers will advertise
          a logical size of 1920×1080,

        - A compositor using a fractional scale of 1.5 will advertise a logical
          size of 2560×1440.

        For example, for a :class:`~pywayland.protocol.wayland.WlOutput` mode
        1920×1080 and a 90 degree rotation, the compositor will advertise a
        logical size of 1080x1920.

        The logical_size event is sent after creating an xdg_output (see
        xdg_output_manager.get_xdg_output) and whenever the logical size of the
        output changes, either as a result of a change in the applied scale or
        because of a change in the corresponding output mode(see
        :func:`WlOutput.mode() <pywayland.protocol.wayland.WlOutput.mode>`) or
        transform (see :func:`WlOutput.transform()
        <pywayland.protocol.wayland.WlOutput.transform>`).

        :param width:
            width in global compositor space
        :type width:
            `ArgumentType.Int`
        :param height:
            height in global compositor space
        :type height:
            `ArgumentType.Int`
        """
        self._post_event(1, width, height)

    @ZxdgOutputV1.event()
    def done(self):
        """All information about the output have been sent

        This event is sent after all other properties of an xdg_output have
        been sent.

        This allows changes to the xdg_output properties to be seen as atomic,
        even if they happen via multiple events.

        For objects version 3 onwards, this event is deprecated. Compositors
        are not required to send it anymore and must send
        :func:`WlOutput.done() <pywayland.protocol.wayland.WlOutput.done>`
        instead.
        """
        self._post_event(2)

    @ZxdgOutputV1.event(
        Argument(ArgumentType.String),
        version=2,
    )
    def name(self, name):
        """Name of this output

        Many compositors will assign names to their outputs, show them to the
        user, allow them to be configured by name, etc. The client may wish to
        know this name as well to offer the user similar behaviors.

        The naming convention is compositor defined, but limited to
        alphanumeric characters and dashes (-). Each name is unique among all
        :class:`~pywayland.protocol.wayland.WlOutput` globals, but if a
        :class:`~pywayland.protocol.wayland.WlOutput` global is destroyed the
        same name may be reused later. The names will also remain consistent
        across sessions with the same hardware and software configuration.

        Examples of names include 'HDMI-A-1', 'WL-1', 'X11-1', etc. However, do
        not assume that the name is a reflection of an underlying DRM
        connector, X11 connection, etc.

        The name event is sent after creating an xdg_output (see
        xdg_output_manager.get_xdg_output). This event is only sent once per
        xdg_output, and the name does not change over the lifetime of the
        :class:`~pywayland.protocol.wayland.WlOutput` global.

        :param name:
            output name
        :type name:
            `ArgumentType.String`
        """
        self._post_event(3, name)

    @ZxdgOutputV1.event(
        Argument(ArgumentType.String),
        version=2,
    )
    def description(self, description):
        """Human-readable description of this output

        Many compositors can produce human-readable descriptions of their
        outputs.  The client may wish to know this description as well, to
        communicate the user for various purposes.

        The description is a UTF-8 string with no convention defined for its
        contents. Examples might include 'Foocorp 11" Display' or 'Virtual X11
        output via :1'.

        The description event is sent after creating an xdg_output (see
        xdg_output_manager.get_xdg_output) and whenever the description
        changes. The description is optional, and may not be sent at all.

        For objects of version 2 and lower, this event is only sent once per
        xdg_output, and the description does not change over the lifetime of
        the :class:`~pywayland.protocol.wayland.WlOutput` global.

        :param description:
            output description
        :type description:
            `ArgumentType.String`
        """
        self._post_event(4, description)


class ZxdgOutputV1Global(Global):
    interface = ZxdgOutputV1


ZxdgOutputV1._gen_c()
ZxdgOutputV1.proxy_class = ZxdgOutputV1Proxy
ZxdgOutputV1.resource_class = ZxdgOutputV1Resource
ZxdgOutputV1.global_class = ZxdgOutputV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.server.client import Client
from pywayland.server.display import Display
from pywayland.server.listener import Listener

import socket


def destroy_notify_a(*args):
    global a
    a = 1


def destroy_notify_b(*args):
    global b
    b = 1


def test_client_destroy_listener():
    global a, b
    s1, s2 = socket.socketpair(socket.AF_UNIX, socket.SOCK_STREAM, 0)
    a = 0
    b = 0

    display = Display()
    client = Client(display, s1.fileno())

    destroy_listener_a = Listener(destroy_notify_a)
    destroy_listener_b = Listener(destroy_notify_b)

    client.add_destroy_listener(destroy_listener_a)
    client.add_destroy_listener(destroy_listener_b)

    destroy_listener_a.remove()

    client.destroy()

    assert a == 0
    assert b == 1



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Type, TYPE_CHECKING

from pywayland import ffi, lib
from pywayland.dispatcher import Dispatcher
from pywayland.utils import ensure_valid

if TYPE_CHECKING:
    from .interface import Interface  # noqa: F401


class Proxy:
    interface: Type["Interface"]

    def __init__(self, ptr, display=None):
        """Represents a protocol object on the client side.

        A :class:`Proxy` acts as a client side proxy to an object existing in
        the compositor.  Events coming from the compositor are also handled by
        the proxy, which will in turn call the handler set with
        :func:`Proxy.add_listener`.
        """
        self.user_data = None
        self.dispatcher = Dispatcher(self.interface.events)

        # This should only be true for wl_display proxies, as they will
        # initialize its pointer on a `.connect()` call
        if ptr is None:
            self._ptr = ptr
            self._display = self
            return

        self._display = display

        # parent display is the root-most client Display object, all proxies
        # should keep the display alive
        if display is None:
            raise ValueError(
                "Non-Display Proxy objects must be associated to a Display"
            )
        display._children.add(self)

        if ptr == ffi.NULL:
            raise RuntimeError("Got a null pointer for the proxy")

        # note that even though we cast to a proxy here, the ptr may be a
        # wl_display, so the methods must still cast to 'struct wl_proxy *'
        ptr = ffi.cast("struct wl_proxy *", ptr)
        self._ptr = ffi.gc(ptr, lib.wl_proxy_destroy)

        self._handle = ffi.new_handle(self)
        lib.wl_proxy_add_dispatcher(
            self._ptr, lib.dispatcher_func, self._handle, ffi.NULL
        )

        self.interface.registry[self._ptr] = self

    @property
    def destroyed(self) -> bool:
        """Determine if proxy has been destroyed

        Returns true if the proxy has been destroyed.
        """
        return self._ptr is None

    def __del__(self) -> None:
        self._destroy()

    def _destroy(self) -> None:
        """Frees the pointer associated with the Proxy"""
        if self._ptr is not None:
            if self._display._ptr is not None:
                ffi.release(self._ptr)
            else:
                self._ptr = ffi.gc(self._ptr, None)
            self._ptr = None

    destroy = _destroy

    @ensure_valid
    def _marshal(self, opcode, *args):
        """Marshal the given arguments into the Wayland wire format"""
        # Create a wl_argument array
        args_ptr = self.interface.requests[opcode].arguments_to_c(*args)

        # Write the event into the connection queue
        proxy = ffi.cast("struct wl_proxy *", self._ptr)
        lib.wl_proxy_marshal_array(proxy, opcode, args_ptr)

    @ensure_valid
    def _marshal_constructor(self, opcode, interface, *args):
        """Marshal the given arguments into the Wayland wire format for a constructor"""
        # Create a wl_argument array
        args_ptr = self.interface.requests[opcode].arguments_to_c(*args)

        # Write the event into the connection queue and build a new proxy from the given args
        proxy = ffi.cast("struct wl_proxy *", self._ptr)
        proxy_ptr = lib.wl_proxy_marshal_array_constructor(
            proxy, opcode, args_ptr, interface._ptr
        )

        return interface.proxy_class(proxy_ptr, self._display)



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from pywayland.protocol_core import Global, Interface, Proxy, Resource


class WlBuffer(Interface):
    """Content for a :class:`~pywayland.protocol.wayland.WlSurface`

    A buffer provides the content for a
    :class:`~pywayland.protocol.wayland.WlSurface`. Buffers are created through
    factory interfaces such as wl_drm,
    :class:`~pywayland.protocol.wayland.WlShm` or similar. It has a width and a
    height and can be attached to a
    :class:`~pywayland.protocol.wayland.WlSurface`, but the mechanism by which
    a client provides and updates the contents is defined by the buffer factory
    interface.
    """

    name = "wl_buffer"
    version = 1


class WlBufferProxy(Proxy):
    interface = WlBuffer

    @WlBuffer.request()
    def destroy(self):
        """Destroy a buffer

        Destroy a buffer. If and how you need to release the backing storage is
        defined by the buffer factory interface.

        For possible side-effects to a surface, see :func:`WlSurface.attach()
        <pywayland.protocol.wayland.WlSurface.attach>`.
        """
        self._marshal(0)
        self._destroy()


class WlBufferResource(Resource):
    interface = WlBuffer

    @WlBuffer.event()
    def release(self):
        """Compositor releases buffer

        Sent when this :class:`WlBuffer` is no longer used by the compositor.
        The client is now free to reuse or destroy this buffer and its backing
        storage.

        If a client receives a release event before the frame callback
        requested in the same :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` that attaches this
        :class:`WlBuffer` to a surface, then the client is immediately free to
        reuse the buffer and its backing storage, and does not need a second
        buffer for the next surface content update. Typically this is possible,
        when the compositor maintains a copy of the
        :class:`~pywayland.protocol.wayland.WlSurface` contents, e.g. as a GL
        texture. This is an important optimization for GL(ES) compositors with
        :class:`~pywayland.protocol.wayland.WlShm` clients.
        """
        self._post_event(0)


class WlBufferGlobal(Global):
    interface = WlBuffer


WlBuffer._gen_c()
WlBuffer.proxy_class = WlBufferProxy
WlBuffer.resource_class = WlBufferResource
WlBuffer.global_class = WlBufferGlobal



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import Optional, Type, TYPE_CHECKING

from pywayland import ffi, lib
from pywayland.dispatcher import Dispatcher
from pywayland.utils import ensure_valid
from pywayland.server.client import Client

if TYPE_CHECKING:
    from .interface import Interface  # noqa: F401


class Resource:
    """A server-side Interface object for the client

    Not created directly, created from the
    :class:`~pywayland.interface.Interface` object.

    :param client: The client that the Resource is for
    :type client: :class:`~pywayland.server.Client` or cdata for ``wl_client *``
    :param version: The version to use for the
                    :class:`~pywayland.interface.Interface`, uses current
                    version if not specified
    :type version: `int`
    :param id: The id for the item
    :type id: `int`
    """

    interface: Type["Interface"]

    def __init__(self, client, version=None, id=0) -> None:
        if version is None:
            version = self.interface.version

        self.version = version
        self.dispatcher = Dispatcher(self.interface.requests, destructor=True)

        if isinstance(client, Client):
            client_ptr = client._ptr
        else:
            client_ptr = client
        assert client_ptr is not None

        self._ptr: Optional["ffi.ResourceCData"] = lib.wl_resource_create(
            client_ptr, self.interface._ptr, version, id
        )
        self.id = lib.wl_resource_get_id(self._ptr)

        if self.dispatcher is not None:
            self._handle = ffi.new_handle(self)
            lib.wl_resource_set_dispatcher(
                self._ptr,
                lib.dispatcher_func,
                ffi.NULL,
                self._handle,
                lib.resource_destroy_func,
            )

    def destroy(self) -> None:
        """Destroy the Resource"""
        if self._ptr:
            lib.wl_resource_destroy(self._ptr)
            self._ptr = None

    @ensure_valid
    def add_destroy_listener(self, listener) -> None:
        """Add a listener for the destroy signal

        :param listener: The listener object
        :type listener: :class:`~pywayland.server.Listener`
        """
        assert self._ptr is not None
        lib.wl_resource_add_destroy_listener(self._ptr, listener._ptr)

    @ensure_valid
    def _post_event(self, opcode, *args) -> None:
        # Create wl_argument array
        args_ptr = self.interface.events[opcode].arguments_to_c(*args)
        # Make the cast to a wl_resource
        assert self._ptr is not None
        resource: ffi.ResourceCData = ffi.cast("struct wl_resource *", self._ptr)  # type: ignore[assignment]

        lib.wl_resource_post_event_array(resource, opcode, args_ptr)

    @ensure_valid
    def _post_error(self, code, msg="") -> None:
        assert self._ptr is not None
        lib.wl_resource_post_error(self._ptr, code, msg.encode())



# This file has been autogenerated by the pywayland scanner

# Copyright © 2014      Jonas Ådahl
# Copyright © 2015      Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlRegion


class ZwpLockedPointerV1(Interface):
    """Receive relative pointer motion events

    The wp_locked_pointer interface represents a locked pointer state.

    While the lock of this object is active, the
    :class:`~pywayland.protocol.wayland.WlPointer` objects of the associated
    seat will not emit any :func:`WlPointer.motion()
    <pywayland.protocol.wayland.WlPointer.motion>` events.

    This object will send the event 'locked' when the lock is activated.
    Whenever the lock is activated, it is guaranteed that the locked surface
    will already have received pointer focus and that the pointer will be
    within the region passed to the request creating this object.

    To unlock the pointer, send the destroy request. This will also destroy the
    wp_locked_pointer object.

    If the compositor decides to unlock the pointer the unlocked event is sent.
    See wp_locked_pointer.unlock for details.

    When unlocking, the compositor may warp the cursor position to the set
    cursor position hint. If it does, it will not result in any relative motion
    events emitted via wp_relative_pointer.

    If the surface the lock was requested on is destroyed and the lock is not
    yet activated, the wp_locked_pointer object is now defunct and must be
    destroyed.
    """

    name = "zwp_locked_pointer_v1"
    version = 1


class ZwpLockedPointerV1Proxy(Proxy):
    interface = ZwpLockedPointerV1

    @ZwpLockedPointerV1.request()
    def destroy(self):
        """Destroy the locked pointer object

        Destroy the locked pointer object. If applicable, the compositor will
        unlock the pointer.
        """
        self._marshal(0)
        self._destroy()

    @ZwpLockedPointerV1.request(
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
    )
    def set_cursor_position_hint(self, surface_x, surface_y):
        """Set the pointer cursor position hint

        Set the cursor position hint relative to the top left corner of the
        surface.

        If the client is drawing its own cursor, it should update the position
        hint to the position of its own cursor. A compositor may use this
        information to warp the pointer upon unlock in order to avoid pointer
        jumps.

        The cursor position hint is double buffered. The new hint will only
        take effect when the associated surface gets it pending state applied.
        See :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` for details.

        :param surface_x:
            surface-local x coordinate
        :type surface_x:
            `ArgumentType.Fixed`
        :param surface_y:
            surface-local y coordinate
        :type surface_y:
            `ArgumentType.Fixed`
        """
        self._marshal(1, surface_x, surface_y)

    @ZwpLockedPointerV1.request(
        Argument(ArgumentType.Object, interface=WlRegion, nullable=True),
    )
    def set_region(self, region):
        """Set a new lock region

        Set a new region used to lock the pointer.

        The new lock region is double-buffered. The new lock region will only
        take effect when the associated surface gets its pending state applied.
        See :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` for details.

        For details about the lock region, see wp_locked_pointer.

        :param region:
            region of surface
        :type region:
            :class:`~pywayland.protocol.wayland.WlRegion` or `None`
        """
        self._marshal(2, region)


class ZwpLockedPointerV1Resource(Resource):
    interface = ZwpLockedPointerV1

    @ZwpLockedPointerV1.event()
    def locked(self):
        """Lock activation event

        Notification that the pointer lock of the seat's pointer is activated.
        """
        self._post_event(0)

    @ZwpLockedPointerV1.event()
    def unlocked(self):
        """Lock deactivation event

        Notification that the pointer lock of the seat's pointer is no longer
        active. If this is a oneshot pointer lock (see
        wp_pointer_constraints.lifetime) this object is now defunct and should
        be destroyed. If this is a persistent pointer lock (see
        wp_pointer_constraints.lifetime) this pointer lock may again reactivate
        in the future.
        """
        self._post_event(1)


class ZwpLockedPointerV1Global(Global):
    interface = ZwpLockedPointerV1


ZwpLockedPointerV1._gen_c()
ZwpLockedPointerV1.proxy_class = ZwpLockedPointerV1Proxy
ZwpLockedPointerV1.resource_class = ZwpLockedPointerV1Resource
ZwpLockedPointerV1.global_class = ZwpLockedPointerV1Global
